//! # Export account API entry
//!
//! @author Pierre Hubert
use std::collections::HashMap;

use serde::Serialize;

use crate::api_data::comment_api::CommentAPI;
use crate::api_data::conversation_api::ConversationAPI;
use crate::api_data::conversation_message_api::ConversationMessageAPI;
use crate::api_data::movie_api::MovieAPI;
use crate::api_data::post_api::PostAPI;
use crate::api_data::survey_response_api::SurveyResponseAPI;
use crate::api_data::user_info::APIUserInfo;
use crate::api_data::user_like_api::UserLikeAPI;
use crate::data::account_export::AccountExport;
use crate::data::error::ResultBoxError;

#[derive(Serialize)]
#[allow(non_snake_case)]
pub struct AccountExportAPI {
    userID: u64,
    advanced_info: APIUserInfo,
    posts: Vec<PostAPI>,
    comments: Vec<CommentAPI>,
    likes: Vec<UserLikeAPI>,
    survey_responses: Vec<SurveyResponseAPI>,
    movies: Vec<MovieAPI>,
    all_conversation_messages: Vec<ConversationMessageAPI>,
    conversations_list: Vec<ConversationAPI>,
    conversations_messages: HashMap<u64, Vec<ConversationMessageAPI>>,
}

impl AccountExportAPI {
    pub fn new(export: &AccountExport) -> ResultBoxError<AccountExportAPI> {
        let curr_user_id = &export.user.id;
        let export = AccountExportAPI {
            userID: curr_user_id.id(),
            advanced_info: APIUserInfo::new_advanced_info(&curr_user_id.as_option(), &export.user)?,
            posts: PostAPI::for_list(&export.posts, curr_user_id.as_option())?,
            comments: CommentAPI::for_list(&export.comments, &curr_user_id.as_option())?,
            likes: UserLikeAPI::for_list(&export.likes),
            survey_responses: SurveyResponseAPI::for_list(&export.survey_responses),
            movies: MovieAPI::for_list(&export.movies),
            all_conversation_messages: ConversationMessageAPI::for_list(&export.all_conversation_messages),
            conversations_list: ConversationAPI::for_list(&export.conversations),
            conversations_messages: export.conversation_messages
                .iter()
                .map(|r| (r.0.clone(), ConversationMessageAPI::for_list(r.1)))
                .collect(),
        };

        Ok(export)
    }
}